searchData={"items":[{"type":"module","title":"FLAMEK8sBackend","doc":"Kubernetes Backend implementation.\n\n#","ref":"FLAMEK8sBackend.html"},{"type":"module","title":"Usage - FLAMEK8sBackend","doc":"Configure the flame backend in our configuration or application setup:\n\n```\n# application.ex\nchildren = [\n  {FLAME.Pool,\n    name: MyApp.SamplePool,\n    backend: FLAMEK8sBackend,\n    min: 0,\n    max: 10,\n    max_concurrency: 5,\n    idle_shutdown_after: 30_000}\n]\n```\n\n### Options\n\nThe following backend options are supported:\n\n  * `:app_container_name` - If your application pod runs multiple containers\n    (initContainers excluded), use this option to pass the name of the\n    container running this application. If not given, the first container\n    in the list of containers is used to lookup the contaienr image, env vars\n    and resources to be used for the runner pods.\n\n  * `:omit_owner_reference` - If true, no ownerReferences are configured on\n    the runner pods. Defaults to `false`\n\n  * `:runner_pod_tpl` - If given, controls how the runner pod manifest is\n    generated. Can be a function of type\n    `t:FLAMEK8sBackend.RunnerPodTemplate.callback/0` or a struct of type\n    `t:FLAMEK8sBackend.RunnerPodTemplate.t/0`.\n    A callback receives the manifest of the parent pod as a map and should\n    return the runner pod's manifest as a map().\n    If a struct is given, the runner pod's manifest will be generated with\n    values from the struct if given or from the parent pod if omitted.\n    If this option is omitted, the parent pod's `env` and `resources`\n    are used for the runner pod.\n    See `FLAMEK8sBackend.RunnerPodTemplate` for more infos.\n\n  * `:log` - The log level to use for verbose logging. Defaults to `false`.\n\n#","ref":"FLAMEK8sBackend.html#module-usage"},{"type":"module","title":"Prerequisites - FLAMEK8sBackend","doc":"In order for this to work, your application needs to meet some requirements.\n\n##","ref":"FLAMEK8sBackend.html#module-prerequisites"},{"type":"module","title":"Env Variables - FLAMEK8sBackend","doc":"In order for the backend to be able to get informations from your pod and use\nthem for  the runner pods (e.g. env variables), you have to define `POD_NAME`\nand `POD_NAMESPACE` environment variables on your pod.\n\n```yaml\napiVersion: apps/v1\nkind: Deployment\nspec:\nselector:\n  matchLabels:\n    app: myapp\ntemplate:\n  spec:\n    containers:\n      - env:\n        - name: POD_NAME\n          valueFrom:\n            fieldRef:\n              apiVersion: v1\n              fieldPath: metadata.name\n        - name: POD_NAMESPACE\n          valueFrom:\n            fieldRef:\n              apiVersion: v1\n              fieldPath: metadata.namespace\n```\n\n##","ref":"FLAMEK8sBackend.html#module-env-variables"},{"type":"module","title":"RBAC - FLAMEK8sBackend","doc":"Your application needs run as a service account with permissions to manage\npods. This is a simple\n\n```yaml\n---\napiVersion: v1\nkind: ServiceAccount\nmetadata:\n  name: myapp\n  namespace: app-namespace\n---\napiVersion: rbac.authorization.k8s.io/v1\nkind: Role\nmetadata:\n  namespace: app-namespace\n  name: pod-mgr\nrules:\n- apiGroups: [\"\"]\n  resources: [\"pods\"]\n  verbs: [\"create\", \"get\", \"list\", \"delete\", \"patch\"]\n---\napiVersion: rbac.authorization.k8s.io/v1\nkind: RoleBinding\nmetadata:\n  name: myapp-pod-mgr\n  namespace: app-namespace\nsubjects:\n- kind: ServiceAccount\n  name: myapp\n  namespace: app-namespace\nroleRef:\n  kind: Role\n  name: pod-mgr\n  apiGroup: rbac.authorization.k8s.io\n---\napiVersion: apps/v1\nkind: Deployment\nspec:\n  template:\n    spec:\n      serviceAccountName: my-app\n```\n\n##","ref":"FLAMEK8sBackend.html#module-rbac"},{"type":"module","title":"Clustering - FLAMEK8sBackend","doc":"Your application needs to be able to form a cluster with your runners. Define\n`POD_IP`, `RELEASE_DISTRIBUTION` and `RELEASE_NODE` environment variables on\nyour pods as follows:\n\n```yaml\napiVersion: apps/v1\nkind: Deployment\nspec:\ntemplate:\n  spec:\n    containers:\n      - env:\n        - name: POD_IP\n          valueFrom:\n            fieldRef:\n              apiVersion: v1\n              fieldPath: status.podIP\n        - name: RELEASE_DISTRIBUTION\n          value: name\n        - name: RELEASE_NODE\n          value: my_app@$(POD_IP)\n```","ref":"FLAMEK8sBackend.html#module-clustering"},{"type":"module","title":"FLAMEK8sBackend.RunnerPodTemplate","doc":"This module is responsible for generating the manifest for the runner pods.\nThe manifest can be overridden using the `runner_pod_tpl` option on\n`FLAMEK8sBackend`.\n\n#","ref":"FLAMEK8sBackend.RunnerPodTemplate.html"},{"type":"module","title":"Simple Use Case - FLAMEK8sBackend.RunnerPodTemplate","doc":"By default, `resources` and `env` variables are copied from the parent pod.\nUsing the `runner_pod_tpl` option on the `FLAMEK8sBackend`, you can add\nadditional environment variables or set different `resources`. You would do\nthis by setting the `runner_pod_tpl` to a struct of type\n`t:FLAMEK8sBackend.RunnerPodTemplate.t/0` as follows:\n\n```\n# application.ex\nalias FLAMEK8sBackend.RunnerPodTemplate\n\nchildren = [\n  {FLAME.Pool,\n    name: MyApp.SamplePool,\n    backend: {FLAMEK8sBackend,\n      runner_pod_tpl: %RunnerPodTemplate{\n        env: [%{\"name\" => \"FOO\", \"value\" => \"bar\"}],\n        resources: %{\n          requests: %{\"memory\" => \"256Mi\", \"cpu\" => \"100m\"},\n          limimts: %{\"memory\" => \"256Mi\", \"cpu\" => \"400m\"}\n        }\n      }\n    },\n    # other opts\n  }\n]\n# ...\n```\n\n#","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#module-simple-use-case"},{"type":"module","title":"Advanced Use Case - Passing a Callback Function - FLAMEK8sBackend.RunnerPodTemplate","doc":"In some cases you might need advanced control over the runner pod manifest.\nMaybe you want to set node affinity because you need your runners to run on\nnodes with GPUs or you need additional volumes etc. In this case, you can pass\na callback via `runner_pod_tpl` option to the `FLAMEK8sBackend`.\n\nThe callback has to be of type `t:FLAMEK8sBackend.RunnerPodTemplate.callback/0`.\nThe callback will be called with the manifest of the parent pod which can be\nused to extract information. It should return a pod template as a map\n\nDefine a callback, e.g. in a separate module:\n\n```\ndefmodule MyApp.FLAMERunnerPodTemplate do\n  def runner_pod_manifest(parent_pod_manifest) do\n    %{\n      \"metadata\" => %{\n        # namespace, labels, ownerReferences,...\n      },\n      \"spec\" => %{\n        \"containers\" => [\n          %{\n            # container definition\n          }\n        ]\n      }\n    }\n  end\nend\n```\n\nRegister the backend:\n\n```\n# application.ex\n# ...\n\nchildren = [\n  {FLAME.Pool,\n    name: MyApp.SamplePool,\n    backend: {FLAMEK8sBackend, runner_pod_tpl: &MyApp.FLAMERunnerPodTemplate.runner_pod_manifest/1},\n    # other opts\n  }\n]\n# ...\n```\n\n> ##","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#module-advanced-use-case-passing-a-callback-function"},{"type":"module","title":"Predefined Values {: .warning} - FLAMEK8sBackend.RunnerPodTemplate","doc":">\n> Note that the following values are controlled by the backend and, if set by\n> your callback function, are going to be overwritten:\n>\n>   * `apiVersion` and `Kind` of the resource (set to `v1/Pod`)\n>   * The pod's and container's names (set to a combination of the parent pod's\n>     name and a random id)\n>   * The `restartPolicy` (set to `Never`)\n>   * The container `image` (set to the image of the parent pod's app container)\n\n#","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#module-predefined-values-warning"},{"type":"module","title":"Options - FLAMEK8sBackend.RunnerPodTemplate","doc":"* `:omit_owner_reference` - Omit generating and appending the parent pod as\n  `ownerReference` to the runner pod's metadata\n\n* `:app_container_name` - name of the container running this application.\n  By default, the first container in the list of containers is used.","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#module-options"},{"type":"function","title":"FLAMEK8sBackend.RunnerPodTemplate.manifest/4","doc":"Generates the POD manifest using information from the parent pod\nand the `runner_pod_tpl` option.","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#manifest/4"},{"type":"type","title":"FLAMEK8sBackend.RunnerPodTemplate.callback/0","doc":"","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#t:callback/0"},{"type":"type","title":"FLAMEK8sBackend.RunnerPodTemplate.parent_pod_manifest/0","doc":"","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#t:parent_pod_manifest/0"},{"type":"type","title":"FLAMEK8sBackend.RunnerPodTemplate.t/0","doc":"","ref":"FLAMEK8sBackend.RunnerPodTemplate.html#t:t/0"},{"type":"extras","title":"FLAMEK8sBakend","doc":"# FLAMEK8sBakend\n\nA [FLAME](https://github.com/phoenixframework/flame/tree/main) Backend for\nKubernetes. Manages pods as runners in the cluster the app is running in.\n\n[![Module Version](https://img.shields.io/hexpm/v/flame_k8s_backend.svg)](https://hex.pm/packages/flame_k8s_backend)\n[![Last Updated](https://img.shields.io/github/last-commit/mruoss/flame_k8s_backend.svg)](https://github.com/mruoss/flame_k8s_backend/commits/main)\n\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/flame_k8s_backend/)\n[![Total Download](https://img.shields.io/hexpm/dt/flame_k8s_backend.svg)](https://hex.pm/packages/flame_k8s_backend)\n[![License](https://img.shields.io/hexpm/l/flame_k8s_backend.svg)](https://github.com/mruoss/flame_k8s_backend/blob/main/LICENSE)\n\nThe current implementation is very basic and more like a proof of concept.\nMore configuration options (resources, etc.) will follow.","ref":"readme.html"},{"type":"extras","title":"Installation - FLAMEK8sBakend","doc":"```elixir\ndef deps do\n  [\n    {:flame_k8s_backend, \"~> 0.3.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - FLAMEK8sBakend","doc":"Configure the flame backend in our configuration or application setup:\n\n```elixir\n  # application.ex\n  children = [\n    {FLAME.Pool,\n      name: MyApp.SamplePool,\n      backend: FLAMEK8sBackend,\n      min: 0,\n      max: 10,\n      max_concurrency: 5,\n      idle_shutdown_after: 30_000,\n      log: :debug}\n  ]\n```","ref":"readme.html#usage"},{"type":"extras","title":"Prerequisites - FLAMEK8sBakend","doc":"#","ref":"readme.html#prerequisites"},{"type":"extras","title":"Env Variables - FLAMEK8sBakend","doc":"In order for the runners to be able to join the cluster, you need to configure\na few environment variables on your pod/deployment:\n\nThe `POD_NAME` and `POD_NAMESPACE` are used by the backend to get informations\nfrom your pod and use them for the runner pods (e.g. env variables).\n\n```yaml\napiVersion: apps/v1\nkind: Deployment\nspec:\n  selector:\n    matchLabels:\n      app: myapp\n  template:\n    spec:\n      containers:\n        - env:\n            - name: POD_NAME\n              valueFrom:\n                fieldRef:\n                  apiVersion: v1\n                  fieldPath: metadata.name\n            - name: POD_NAMESPACE\n              valueFrom:\n                fieldRef:\n                  apiVersion: v1\n                  fieldPath: metadata.namespace\n```\n\n#","ref":"readme.html#env-variables"},{"type":"extras","title":"RBAC - FLAMEK8sBakend","doc":"Your application needs run as a service account with permissions to manage\npods:\n\n```yaml\n---\napiVersion: v1\nkind: ServiceAccount\nmetadata:\n  name: myapp\n  namespace: app-namespace\n---\napiVersion: rbac.authorization.k8s.io/v1\nkind: Role\nmetadata:\n  namespace: app-namespace\n  name: pod-mgr\nrules:\n  - apiGroups: [\"\"]\n    resources: [\"pods\"]\n    verbs: [\"create\", \"get\", \"list\", \"delete\", \"patch\"]\n---\napiVersion: rbac.authorization.k8s.io/v1\nkind: RoleBinding\nmetadata:\n  name: myapp-pod-mgr\n  namespace: app-namespace\nsubjects:\n  - kind: ServiceAccount\n    name: myapp\n    namespace: app-namespace\nroleRef:\n  kind: Role\n  name: pod-mgr\n  apiGroup: rbac.authorization.k8s.io\n---\napiVersion: apps/v1\nkind: Deployment\nspec:\n  template:\n    spec:\n      serviceAccountName: my-app\n```\n\n#","ref":"readme.html#rbac"},{"type":"extras","title":"Clustering - FLAMEK8sBakend","doc":"Your application needs to be able to form a cluster with your runners. Define\n`POD_IP`, `RELEASE_DISTRIBUTION` and `RELEASE_NODE` environment variables on\nyour pods as follows:\n\n```yaml\napiVersion: apps/v1\nkind: Deployment\nspec:\n  template:\n    spec:\n      containers:\n        - env:\n            - name: POD_IP\n              valueFrom:\n                fieldRef:\n                  apiVersion: v1\n                  fieldPath: status.podIP\n            - name: RELEASE_DISTRIBUTION\n              value: name\n            - name: RELEASE_NODE\n              value: my_app@$(POD_IP)\n```","ref":"readme.html#clustering"},{"type":"extras","title":"How it works - FLAMEK8sBakend","doc":"The FLAME Kubernetes backend first queries the Kubernetes API server to extract\ninformation from the running Pod like the container image, resource requests and\nlimits, environment variables etc. This information is then used to build the\nmanifest for the runner pod. The backend then sends the resulting manifest to\nthe API server in order to spin up a runner pod.","ref":"readme.html#how-it-works"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/).","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"<!-- Add your changelog entry to the relevant subsection -->\n\n<!-- #","ref":"changelog.html#unreleased"},{"type":"extras","title":"Added | Changed | Deprecated | Removed | Fixed | Security --> - Changelog","doc":"<!--------------------- Don't add new entries after this line --------------------->\n\n## [0.3.1] - 2024-01-28\n\n#","ref":"changelog.html#added-changed-deprecated-removed-fixed-security"},{"type":"extras","title":"Changed - Changelog","doc":"- Use `:cacertfile` insead of `:cacerts` in `:transport_options` and let the OTP process the certificate - [#8](https://github.com/mruoss/flame_k8s_backend/pull/8)\n- Dependency Updates\n\n## [0.3.0] - 2023-12-19\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Remove`:insecure_skip_tls_verify` option and use a custom `match_fun` instead to work around failing hostname verification for IP addresses. - [#5](https://github.com/mruoss/flame_k8s_backend/pull/5)\n\n## [0.2.3] - 2023-12-15\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `runner_pod_tpl` option for better control over the runner pod manifest - [#2](https://github.com/mruoss/flame_k8s_backend/pull/2)\n- Basic integration test\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Delete pod when shutting down the runner.\n\n## [0.2.2] - 2023-12-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Don't crash the runner if the `:log` option is not set (or set to `false`)\n\n## [0.2.1] - 2023-12-11\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- ENV var `DRAGONFLY_PARENT` was renamed to `FLAME_PARENT` in commit [9c2e65cc](https://github.com/phoenixframework/flame/commit/9c2e65ccd2c55514a473ad6ed986326576687064)\n\n## [0.2.0] - 2023-12-10\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Replace `k8s` lib with a lightweight Kubernetes client implementation.\n\n## [0.1.0] - 2023-12-09\n\n- Very early stage implementation of a Kubernetes backend.","ref":"changelog.html#changed"}],"content_type":"text/markdown"}